/****************************************************************************/
/*  Copyright 2005, Trustees of Indiana University                          */
/*  Author: Sidharth Thakur                                                 */
/*  Last modified: 14 January 2006                                          */
/*  License available at: <belt-trick-* folder>/docs/downloads/License.txt  */
/****************************************************************************/
#ifndef _STRUCTS_H
#define _STRUCTS_H

///////////////////////////////////////////////////////////////////////////////
//
// $Header:  structs.h$
//
// NAME: none
//
// BASE  CLASSES: none
//  
// PURPOSE: Global include for data structures
//
// AUTHOR: Sidharth Thakur
//
///////////////////////////////////////////////////////////////////////////////

#include <assert.h>

#define NMAX 6
#define MAX_POINTS	500

#ifndef M_PI
#define	M_PI						3.14159265358979330846
#endif

#define FALSE                      0
#define TRUE                       1

#define BUFSIZE 512
#define MAX_TEXTURES 10

#define ON  TRUE
#define OFF FALSE
// OPERATIONS
#define sign(A)                 ((A) < 0.0 ? -1.0 : ((A) > 0.0 ? 1.0 : 0.0))
#define Min(A,B)                ((A) <= (B) ? (A) : (B))
#define Max(A,B)                ((A) >= (B) ? (A) : (B))
#define Inbetween(A,X,B)        ((Min(A,B) <= (X)) && ((X) <= Max(A,B)))
#define zap(x)					{ assert(x != NULL); delete(x); x = NULL; }

/* MATH OBJECT TYPE */
enum { 
	MATHOBJ = 200,
	OBJKNOT, 
	OBJCURVE, 
	OBJSPLINE,
	OBJSURFACE,
	OBJBELT
};

/* QUATERNION COMPONENTS */
enum{
	SHOWXYZ,
	SHOWWYZ,
	SHOWWXY,
	SHOWWXZ
};

enum {
	BELT360,
	BELT720
};

enum{
	BELT_PATTERN_SEGMENTS,
	BELT_PATTERN_DOUBLE_SIDED,
	BELT_PATTERN_TEXTURE
};

typedef struct { double d[3]; } Point3;
typedef struct { double d[4]; } Point4;
typedef struct { double d[5]; } Point5;
typedef struct { double d[6]; } Point6;
typedef struct { double d[NMAX]; } PointN;

typedef struct { double d[3]; } Vector3;
typedef struct { double d[4]; } Vector4;
typedef struct { double d[5]; } Vector5;
typedef struct { double d[6]; } Vector6;
typedef struct { double d[NMAX]; } VectorN;

typedef struct { float f[4];   } Color4; 
//typedef struct { double d[4];  } HmgPoint3;
//typedef struct { double d[4];  } HmgVector3;
typedef struct { double d[16]; } Matrix4;
typedef struct { double d[25]; } Matrix5;
typedef struct { double d[36]; } Matrix6;
typedef struct { double d[16]; } MatrixGL;

typedef struct { double d[3]; } Texture;

typedef struct Vertex Vertex;
typedef struct Face Face;
typedef struct Obj Obj;

struct Vertex{
    int       index;             // index of this vertex in current obj 
    Point3    p3;                // 3d coordinates 
    int       depth;             // 4d depth between 0 and 255 
    Point4    p4;                // 4d coordinates 
    //Point5 p5;                 // 5d coordinates 
    //Point6 p6;                 // 6d coordinates 
    //PointN pN;                 // Nd coordinates 
    Vector3   normal ;           // 3d vertex normal 
    Color4    cfront, cback;     // RGBA colors 
    Texture   tex;               // Texture coordinates 
    int       Nfaces;            // number of neighbouring faces 
    Face      **faces;           // list of pointers to neighbouring faces 
};

struct Face {
    int       index;             // index of this face in current obj 
    int       nvert;             // # of vertices in the face (4 in MESH) 
    Vertex ** vertices;          // pointers to the vertices 
    Vector3   normal;            // 3d plane normal 
    Color4    cfront, cback;     // RGBA colors 
};

//@ For: Tube
struct ThickenedFrame 
{
    Vertex    **rpnts;           // Ring points at each vertex 
    Vector3   tangent;           // Tangent vector 
    Vector3   normal;            // Normal vector 
    Vector3   binormal;          // Binormal Vector
    
};

struct ThickenedCurve {

    Vertex   ** vertices;        // Curve vertex 
    ThickenedFrame **tframes;    // Frame at each point on curve 
    int    frame_type;           // Type of Frame: Frenet, Parallel, Quaternion 
    
};


struct Obj {
 int    gformat;        /* FORMAT_OFF, MESH, LIST, FRAMES, DOT, LINES, GLM_OBJ etc.*/
 int    cformat;        /* UNIFORM (0, 1 or 2 colors per obj) */ 
 int    texture_flag;   /* Does user supply texture coordinates for vertices */
 int    vcolor;         /* number of colors supplied for each vertex, 0, 1 or 2 */
 int    fcolor;         /* number of colors supplied for each face, 0, 1 or 2 */
 int    nvertices, nfaces;
 int    nu, nv;         /* sizes of mesh, useful only when format_flag = MESH */
 int    u_flag;         /* part of MESH format */
 int    v_flag;         /* part of MESH format */
 int    size_flag;      /* valid in POINTS/ LINES formats */
 float   size;        

 Vertex   *vlist;
 Face   *flist;         /* in LINES format holds lines */
 double   scale;        /* scale and center are used to fit object */
 Point4   center;       /* into identity sphere */
 Color4   cfront, cback;/* color */

 int    nframes;        /* data for frame mode */
 Obj    **frames;


 //Pointer to curve's tube
 ThickenedCurve **curveTube;

};



typedef struct node{
 double v;
 struct node *next;
} node;

typedef struct {
    int width, height;        // Width and height of the texture
    unsigned char *buffer;    // Pointer to memory that holds the texture information.
} Image;

typedef struct {
 node *begin;
 node *end;
 int count;
} queue;

typedef struct {
        double d[3];
}vector3;


#endif //_STRUCTS_H

